import { type Metadata } from 'next'
import Image from 'next/image'
import Link from 'next/link'
import clsx from 'clsx'
import { Card } from '@/components/Card'

import { Container } from '@/components/Container'
import originalDeedImage from '@/images/original-deed.png'
import { EnvelopeIcon } from '@heroicons/react/24/solid'

function SocialLink({
  className,
  href,
  children,
  icon: Icon,
}: {
  className?: string
  href: string
  icon: React.ComponentType<{ className?: string }>
  children: React.ReactNode
}) {
  return (
    <li className={clsx(className, 'flex')}>
      <Link
        href={href}
        className="group flex text-sm font-medium text-zinc-800 transition hover:text-teal-500 dark:text-zinc-200 dark:hover:text-teal-500"
      >
        <Icon className="h-6 w-6 flex-none fill-zinc-500 transition group-hover:fill-teal-500" />
        <span className="ml-4">{children}</span>
      </Link>
    </li>
  )
}

function TimelineEntry({
  title,
  description,
  event,
  cta,
  href,
  children
}: {
  title: string
  description: string
  event?: string
  cta?: string
  href?: string
  children?: React.ReactNode
}) {
  return (
    <Card as="article">
      <Card.Title as="h3" href={href}>
        {title}
      </Card.Title>
      {event
        ? <Card.Eyebrow decorate>{event}</Card.Eyebrow>
        : null}
      <Card.Description>{description}</Card.Description>
      {cta
        ? <Card.Cta>{cta}</Card.Cta>
        : null}
      {children}
    </Card>
  )
}

function DateListItem({
  year, value
}: {
  year?: string
  value: string
}) {
  return (
    <div className='flex space-x-4'>
      <div className='w-10'>{year}</div>
      <div>{value}</div>
    </div>
  )
}

export const metadata: Metadata = {
  title: 'History',
  description:
    'The history of the West Sound Community Hall on Orcas Island.',
}

export default function About() {
  return (
    <Container className="mt-16 sm:mt-32">
      <div className="grid grid-cols-1 gap-y-16 lg:grid-cols-2 lg:grid-rows-[auto_1fr] lg:gap-y-12">
        <div className="lg:pl-20">
          <div className="max-w-xs px-2.5 lg:max-w-none">
            <Image
              src={originalDeedImage}
              alt=""
              sizes="(min-width: 1024px) 32rem, 20rem"
              className="rotate-3 rounded-2xl bg-zinc-100 object-cover dark:bg-zinc-800"
            />
            <p className="text-sm text-zinc-400 dark:text-zinc-500">
              Original property deed from Alexander Chalmers.
            </p>
          </div>
        </div>
        <div className="lg:order-first lg:row-span-2">
          <h1 className="text-4xl font-bold tracking-tight text-zinc-800 dark:text-zinc-100 sm:text-5xl">
            Hall History
          </h1>
          <div className="mt-6 space-y-7 text-base text-zinc-600 dark:text-zinc-400">
            <p>
              The West Sound Community Hall represents the history and character
              of Orcas Island. Members of the nonprofit West Sound Community Club,
              as stewards of the Hall, strive to maintain the integrity of the
              Hall as a unique gathering place for future generations.
            </p>
            <p>
              In 1902 volunteers began building the Hall with materials supplied by George
              Adkins. The building site was donated two years earlier by Alexander Chalmers.
              The Hall was erected to serve as a central meeting place for residents of the
              West Sound area.
            </p>
            <p>
              Over the years the original one-room schoolhouse design has
              changed to fit the times. A kitchen, stage, and entrance porch
              were added to enhance the building&apos;s function.
            </p>
            <p>
              Numerous organizations have used the Hall for meetings. In
              addition, weddings, concerts, elections, parties, and other events
              have been regularly held in the Hall since 1902.
            </p>
          </div>
          <div className="mt-8 space-y-8">
            <h2 className="text-2xl font-bold tracking-tight text-zinc-800 dark:text-zinc-100 sm:text-5xl">
              Detailed History
            </h2>

            <TimelineEntry
              title="Pre-1900"
              description='A Lummi village named "Elelung" occupied the land. The property is part of a documented archaeological site as evidenced by a shell midden under the Hall.'
            />
            <TimelineEntry
              title="1900 - February 21st"
              description='Alexander Chalmers donated "property to be used for a site for a public hall" to the West Sound Hall Company. The donated land was originally part of a large parcel homesteaded by Peter LaPlante in 1884.'
            />
            <TimelineEntry
              title='1902 - 1903'
              description='The West Sound Community Hall was constructed. Funds for building materials were donated by George Adkins. The Hall was constructed by Adkins, Omer Freel, Joe Verrier, Peter LaPlante, Gus Smedberg and others. Mr. Adkins, who moved to West Sound in 1896, founded the West Sound Trading Company which was located on a wharf opposite the Hall.'
            />
            <TimelineEntry
              title='1904'
              description='The Chase brothers, who owned the Chase Bros. Mill in West Sound, donated lumber and built an addition for a stage on the north end of the Hall.'
            />
            <TimelineEntry
              title='1903 - 1935'
              description='During this period the following organizations were regular users of the Hall:'
            >
              <div className="relative z-10 ml-4 mt-2 text-sm text-zinc-600 dark:text-zinc-400">
                <DateListItem
                  year='1903'
                  value='Women&apos;s Christian Temperance Union Woodmen Lodge'
                />
                <DateListItem
                  value='West Sound Literary Society'
                />
                <DateListItem
                  year='1911'
                  value='West Sound Boy&apos;s Band'
                />
                <DateListItem
                  year='1912'
                  value='West Sound Grange (130 members in 1913)'
                />
                <DateListItem
                  value='Odd Fellows'
                />
                <DateListItem
                  value='West Sound Athletic Club'
                />
                <DateListItem
                  year='1913'
                  value='West Sound Orchestra'
                />
                <DateListItem
                  value='West Sound Baseball Team'
                />
                <DateListItem
                  year='1922'
                  value='Farm Bureau'
                />
                <DateListItem
                  year='1925'
                  value='Fidelis Circle'
                />
              </div>
            </TimelineEntry>
            <TimelineEntry
              title='1905 - 1910'
              description='Sometime during this period the Hall was increased in size by adding twelve feet to the south end.'
            />
            <TimelineEntry
              title='1935'
              description='West Sound Hall Company is reorganized as the West Sound Community Club.'
            />
            <TimelineEntry
              title='Sometime after 1935'
              description='The stage, then located at the north end of the hall, was remodeled " to make it as wide as the Hall and 22 feet back providing room for stage, kitchen and rest room - cost $122.85." Logs were salvaged from local beaches and the Thatcher Mill sawed them into lumber for use in remodel.'
            />
            <TimelineEntry
              title='1940&apos;s'
              description='During the Second World War, Fidelis Circle cared for and managed the Hall.'
            />
            <TimelineEntry
              title='Sometime after 1947'
              description='When it was determined the foundation under the stage was failing, the stage was converted to a kitchen and the floor was lowered to match the level of the main Hall floor. The current stage on the east side of the Hall was probably added at the same time.'
            />
            <TimelineEntry
              title='1956'
              description='The Orcas Island Yacht Club was founded and began using the Hall as a clubhouse.'
            />
            <TimelineEntry
              title='1957'
              description='The south entry was moved to the present location on the east side of the Hall. The old entry and scaffolding for the new entry are shown in a photo taken in April 1957. This is the only known photo showing in detail how the exterior of the Hall looked prior to 1957.'
            />
            <TimelineEntry
              title='1972'
              description='Members of the Orcas Island Yacht Club made major repairs to the building structure and the interior was completely remodeled. Prior to the remodel the interior walls were burlap cloth and there was no ceiling in the hall.'
            />
            <TimelineEntry
              title='1979'
              description='The property on which the hall sits was increased in size by donation a strip of land surrounding the hall structure.'
            />
            <TimelineEntry
              title='1981'
              description='Major foundation repairs of a temporary nature were made.'
            />
            <TimelineEntry
              title='1996'
              description='The entry stairs and deck were rebuilt.'
            />
            <TimelineEntry
              title='1999'
              description='Realizing a major renovation of the Hall was needed, the officers of the West Sound Community Club reorganized the club as a nonprofit corporation and applied for 501(c)(3) status. A Centennial Building Committee was created to manage the renovation. The goal was to complete the renovation prior to the 100th anniversary of the Hall&appos;s construction in 1902. Con Russell Construction was selected as the contractor and construction started in February 2000. Major repairs, including a new foundation, metal roof, insulation, and painting the exterior, were completed by October of 2000. To date, the Club has received contributions and pledges of more than $81,000 to pay for the renovation and establish an endowment for future maintenance of the Hall.'
            />
            <TimelineEntry
              title='1999 - October 18th'
              description='The West Sound Community Hall was listed in the Washington Heritage Register.'
            />
            <TimelineEntry
              title='2000 - June 12th'
              description='The Washington Department of Revenue approved the West Sound Community Club&apos;s application for property tax exemption for the West Sound Community Hall as a "public assembly hall".'
            />
            <TimelineEntry
              title='2000 - August 31st'
              description='West Sound Community Hall web site first launched.'
            />
            <TimelineEntry
              title='2001 - April 21st'
              description='A new "West Sound Community Hall" sign was placed on the front of the Hall in a ceremony attended by more than twenty members of the West Sound Community Club. Placement of the sign marked the completion of a major renovation of the Hall started in February 2000.'
            />
          </div>
        </div>
        <div className="lg:pl-20">
          <ul role="list">
            <SocialLink
              href="mailto:contact@westsoundhall.org"
              icon={EnvelopeIcon}
            >
              contact@westsoundhall.org
            </SocialLink>
          </ul>
        </div>
      </div>
    </Container>
  )
}
